import json
from abc import ABC
from requests import request
from urllib.parse import urlencode

from asc.core import log


class APIRequestor(ABC):

    base_api_url:str  # Não informar '/' ao final.

    base_queries:dict = {}
    base_payload:dict = {}
    base_headers:dict = {}

    def mountURL(self,url : str):
        URL = '%s/%s' % (self.base_api_url,url)
        log.debug(f'url={URL}')
        return URL

    def mountHeaders(self,headers:dict={}):
        headers = {
            **{
                k:' '.join(v) if isinstance(v,list) else v \
                    for k,v in self.base_headers.items()
            },
            **headers
        }

        log.debug(headers)
        return headers

    def mountPayload(self,payload:dict={}):
        
        req_payload = self.base_payload or {}

        if payload: 
            req_payload = {
                **req_payload,
                **payload
            }

        log.debug(payload)
        return req_payload

    def toURLQueryString(self,query:dict={}):

        query = {
            **self.base_queries,
            **query
        }

        if query:
            return '?%s' % urlencode(query)

        return ''

    def GET(self, url, query:dict={}):

        request_url = self.mountURL(url + self.toURLQueryString(query))
        request_headers = self.mountHeaders()
        request_payload = self.mountPayload()

        return request(
            "GET", 
            url=request_url,
            headers=request_headers,
            data=request_payload,
            files=[]
        )

    def POST(self, url, data:dict={}):
        jsonData = self.mountPayload(data)

        return request(
            "POST", 
            url=self.mountURL(url), 
            headers=self.mountHeaders(), 
            json=jsonData, 
        )

    def PUT(self, url, data:dict={}):
        jsonData = self.mountPayload(data)

        return request(
            "PUT",
            url=self.mountURL(url),
            headers=self.mountHeaders(),
            json=jsonData,
        )
